@@ -7,12 +7,12 @@ from typing import TYPE_CHECKING
 import bentoml
 from bentoml import Tag
 
-from .torchscript import save_model as script_save_model
-from .torchscript import MODEL_FILENAME
-from ...exceptions import NotFound
 from ...exceptions import MissingDependencyException
+from ...exceptions import NotFound
 from ..models.model import Model
 from .common.pytorch import torch
+from .torchscript import MODEL_FILENAME
+from .torchscript import save_model as script_save_model
 
 if TYPE_CHECKING:
     from ..models.model import ModelSignaturesType
@@ -77,7 +77,7 @@ def load_model(
 
 
 def save_model(
-    name: str,
+    name: Tag | str,
     model: pl.LightningModule,
     *,
     signatures: ModelSignaturesType | None = None,
@@ -163,9 +163,9 @@ def save_model(
 
     script_module = model.to_torchscript()
 
-    assert not isinstance(
-        script_module, dict
-    ), "Saving a dict of pytorch_lightning Module into one BentoModel is not supported"
+    assert not isinstance(script_module, dict), (
+        "Saving a dict of pytorch_lightning Module into one BentoModel is not supported"
+    )
 
     return script_save_model(
         name,
@@ -184,9 +184,9 @@ def get_runnable(bento_model: Model):
     """
     Private API: use :obj:`~bentoml.Model.to_runnable` instead.
     """
-    from .common.pytorch import partial_class
     from .common.pytorch import PytorchModelRunnable
     from .common.pytorch import make_pytorch_runnable_method
+    from .common.pytorch import partial_class
 
     for method_name, options in bento_model.info.signatures.items():
         PytorchModelRunnable.add_method(
