@@ -1,24 +1,25 @@
 from __future__ import annotations
 
-import typing as t
 import logging
+import typing as t
 from abc import ABC
 from abc import abstractmethod
 
 import attr
-from simple_di import inject
 from simple_di import Provide
+from simple_di import inject
 
+from ...exceptions import StateException
+from ..configuration.containers import BentoMLContainer
+from ..models.model import Model
 from ..tag import validate_tag_str
+from ..utils import deprecated
 from ..utils import first_not_none
 from .runnable import Runnable
-from .strategy import Strategy
-from .strategy import DefaultStrategy
-from ...exceptions import StateException
-from ..models.model import Model
-from .runner_handle import RunnerHandle
 from .runner_handle import DummyRunnerHandle
-from ..configuration.containers import BentoMLContainer
+from .runner_handle import RunnerHandle
+from .strategy import DefaultStrategy
+from .strategy import Strategy
 
 if t.TYPE_CHECKING:
     from ...triton import Runner as TritonRunner
@@ -47,6 +48,7 @@ class RunnerMethod(t.Generic[T, P, R]):
     config: RunnableMethodConfig
     max_batch_size: int
     max_latency_ms: int
+    doc: str | None = None
 
     def run(self, *args: P.args, **kwargs: P.kwargs) -> R:
         return self.runner._runner_handle.run_method(self, *args, **kwargs)
@@ -54,6 +56,11 @@ class RunnerMethod(t.Generic[T, P, R]):
     async def async_run(self, *args: P.args, **kwargs: P.kwargs) -> R:
         return await self.runner._runner_handle.async_run_method(self, *args, **kwargs)
 
+    def async_stream(
+        self, *args: P.args, **kwargs: P.kwargs
+    ) -> t.AsyncGenerator[str, None]:
+        return self.runner._runner_handle.async_stream_method(self, *args, **kwargs)
+
 
 def _to_lower_name(name: str) -> str:
     lname = name.lower()
@@ -85,6 +92,7 @@ class AbstractRunner(ABC):
     )
     resource_config: dict[str, t.Any]
     runnable_class: type[Runnable]
+    embedded: bool
 
     @abstractmethod
     def init_local(self, quiet: bool = False) -> None:
@@ -96,15 +104,20 @@ class AbstractRunner(ABC):
         """
 
     @abstractmethod
-    def init_client(self):
+    def init_client(
+        self,
+        handle_class: type[RunnerHandle] | None = None,
+        *args: t.Any,
+        **kwargs: t.Any,
+    ):
         """
         Initialize client for a remote runner instance. To be used within API server instance.
         """
 
 
-@attr.define(slots=False, frozen=True, eq=False)
+@deprecated(suggestion="Please upgrade to new style services.")
+@attr.define(slots=False, frozen=True, eq=False, init=False)
 class Runner(AbstractRunner):
-
     if t.TYPE_CHECKING:
         # This will be set by __init__. This is for type checking only.
         run: t.Callable[..., t.Any]
@@ -113,25 +126,28 @@ class Runner(AbstractRunner):
         # the following annotations hacks around the fact that Runner does not
         # have information about signatures at runtime.
         @t.overload
-        def __getattr__(self, item: t.Literal["__attrs_init__"]) -> t.Callable[..., None]:  # type: ignore
+        def __getattr__(
+            self, item: t.Literal["__attrs_init__"]
+        ) -> t.Callable[..., None]:  # type: ignore
             ...
 
         @t.overload
-        def __getattr__(self, item: t.LiteralString) -> RunnerMethod[t.Any, P, t.Any]:
-            ...
+        def __getattr__(
+            self, item: t.LiteralString
+        ) -> RunnerMethod[t.Any, P, t.Any]: ...
 
-        def __getattr__(self, item: str) -> t.Any:
-            ...
+        def __getattr__(self, item: str) -> t.Any: ...
 
     runner_methods: list[RunnerMethod[t.Any, t.Any, t.Any]]
     scheduling_strategy: type[Strategy]
+    workers_per_resource: int | float = 1
     runnable_init_params: dict[str, t.Any] = attr.field(
         default=None, converter=attr.converters.default_if_none(factory=dict)
     )
     _runner_handle: RunnerHandle = attr.field(init=False, factory=DummyRunnerHandle)
 
     def _set_handle(
-        self, handle_class: type[RunnerHandle], *args: P.args, **kwargs: P.kwargs
+        self, handle_class: type[RunnerHandle], *args: t.Any, **kwargs: t.Any
     ) -> None:
         if not isinstance(self._runner_handle, DummyRunnerHandle):
             raise StateException("Runner already initialized")
@@ -160,11 +176,12 @@ class Runner(AbstractRunner):
         max_batch_size: int | None = None,
         max_latency_ms: int | None = None,
         method_configs: dict[str, dict[str, int]] | None = None,
+        embedded: bool = False,
     ) -> None:
         """
 
         Runner represents a unit of computation that can be executed on a remote Python worker and scales independently
-        See https://docs.bentoml.org/en/latest/concepts/runner.html for more details.
+        See https://docs.bentoml.com/en/latest/concepts/runner.html for more details.
 
         Args:
             runnable_class: Runnable class that can be executed on a remote Python worker.
@@ -182,7 +199,7 @@ class Runner(AbstractRunner):
             method_configs: A dictionary per method config for this given Runner signatures.
 
         Returns:
-            :obj:`bentoml.Runner`: A Runner instance.
+            :obj:`bentoml.legacy.Runner`: A Runner instance.
         """
 
         if name is None:
@@ -227,6 +244,7 @@ class Runner(AbstractRunner):
                 runner=self,
                 name=method_name,
                 config=method.config,
+                doc=method.func.__doc__,
                 max_batch_size=first_not_none(
                     method_max_batch_size,
                     max_batch_size,
@@ -245,8 +263,10 @@ class Runner(AbstractRunner):
             runnable_class=runnable_class,
             runnable_init_params=runnable_init_params,
             resource_config=config["resources"],
+            workers_per_resource=config.get("workers_per_resource", 1),
             runner_methods=list(runner_method_map.values()),
             scheduling_strategy=scheduling_strategy,
+            embedded=embedded,
         )
 
         # Choose the default method:
@@ -305,8 +325,8 @@ class Runner(AbstractRunner):
     def init_client(
         self,
         handle_class: type[RunnerHandle] | None = None,
-        *args: P.args,
-        **kwargs: P.kwargs,
+        *args: t.Any,
+        **kwargs: t.Any,
     ):
         if handle_class is None:
             from .runner_handle.remote import RemoteRunnerClient
@@ -327,6 +347,7 @@ class Runner(AbstractRunner):
         return self.scheduling_strategy.get_worker_count(
             self.runnable_class,
             self.resource_config,
+            self.workers_per_resource,
         )
 
     @property
@@ -335,6 +356,7 @@ class Runner(AbstractRunner):
             worker_id: self.scheduling_strategy.get_worker_env(
                 self.runnable_class,
                 self.resource_config,
+                self.workers_per_resource,
                 worker_id,
             )
             for worker_id in range(self.scheduled_worker_count)
