@@ -1,7 +1,7 @@
 from __future__ import annotations
 
-import typing as t
 import logging
+import typing as t
 from abc import ABC
 from abc import abstractmethod
 from typing import TYPE_CHECKING
@@ -9,9 +9,9 @@ from typing import TYPE_CHECKING
 from ....exceptions import StateException
 
 if TYPE_CHECKING:
+    from ..runner import AbstractRunner
     from ..runner import Runner
     from ..runner import RunnerMethod
-    from ..runner import AbstractRunner
 
     R = t.TypeVar("R")
     P = t.ParamSpec("P")
@@ -22,8 +22,7 @@ logger = logging.getLogger(__name__)
 
 class RunnerHandle(ABC):
     @abstractmethod
-    def __init__(self, runner: AbstractRunner) -> None:
-        ...
+    def __init__(self, runner: AbstractRunner) -> None: ...
 
     @abstractmethod
     async def is_ready(self, timeout: int) -> bool:
@@ -35,8 +34,7 @@ class RunnerHandle(ABC):
         __bentoml_method: RunnerMethod[t.Any, P, R],
         *args: P.args,
         **kwargs: P.kwargs,
-    ) -> R:
-        ...
+    ) -> R: ...
 
     @abstractmethod
     async def async_run_method(
@@ -44,8 +42,15 @@ class RunnerHandle(ABC):
         __bentoml_method: RunnerMethod[t.Any, P, R],
         *args: P.args,
         **kwargs: P.kwargs,
-    ) -> R:
-        ...
+    ) -> R: ...
+
+    @abstractmethod
+    def async_stream_method(
+        self,
+        __bentoml_method: RunnerMethod[t.Any, P, R],
+        *args: P.args,
+        **kwargs: P.kwargs,
+    ) -> t.AsyncGenerator[R, None]: ...
 
 
 class DummyRunnerHandle(RunnerHandle):
@@ -76,3 +81,13 @@ class DummyRunnerHandle(RunnerHandle):
         raise StateException(
             f"Runner is not initialized. Make sure to include '{self!r}' to your service definition."
         )
+
+    def async_stream_method(
+        self,
+        __bentoml_method: RunnerMethod[t.Any, t.Any, t.Any],
+        *args: t.Any,
+        **kwargs: t.Any,
+    ) -> t.AsyncGenerator[t.Any, None]:
+        raise StateException(
+            f"Runner is not initialized. Make sure to include '{self!r}' to your service definition."
+        )
