@@ -1,33 +1,34 @@
 from __future__ import annotations
 
+import asyncio
+import functools
+import logging
 import os
 import sys
 import typing as t
-import asyncio
-import logging
-import functools
 
-from simple_di import inject
 from simple_di import Provide
-from starlette.responses import PlainTextResponse
+from simple_di import inject
 from starlette.exceptions import HTTPException
+from starlette.responses import PlainTextResponse
 
-from ..context import trace_context
-from ..context import InferenceApiContext as Context
 from ...exceptions import BentoMLException
+from ..configuration.containers import BentoMLContainer
+from ..context import trace_context
 from ..server.base_app import BaseAppFactory
 from ..service.service import Service
-from ..configuration.containers import BentoMLContainer
+from ..utils import with_app_arg
 
 if t.TYPE_CHECKING:
-    from starlette.routing import BaseRoute
+    from opentelemetry.sdk.trace import Span
+    from starlette.applications import Starlette
+    from starlette.middleware import Middleware
     from starlette.requests import Request
     from starlette.responses import Response
-    from starlette.middleware import Middleware
-    from starlette.applications import Starlette
-    from opentelemetry.sdk.trace import Span
+    from starlette.routing import BaseRoute
 
     from ..service.inference_api import InferenceAPI
+    from ..types import LifecycleHook
 
 
 logger = logging.getLogger(__name__)
@@ -47,8 +48,8 @@ DEFAULT_INDEX_HTML = """\
     <!-- End Google Tag Manager -->
     <link rel="stylesheet" type="text/css" href="./static_content/swagger-ui.css" />
     <link rel="stylesheet" type="text/css" href="./static_content/index.css" />
-    <link rel="icon" type="image/png" href="./static_content/favicon-32x32.png" sizes="32x32" />
-    <link rel="icon" type="image/png" href="./static_content/favicon-96x96.png" sizes="96x96" />
+    <link rel="icon" type="image/png" href="./static_content/favicon-light-32x32.png" sizes="32x32" media="(prefers-color-scheme: light)" />
+    <link rel="icon" type="image/png" href="./static_content/favicon-dark-32x32.png" sizes="32x32" media="(prefers-color-scheme: dark)" />
   </head>
   <body>
     <!-- Google Tag Manager (noscript) -->
@@ -73,7 +74,7 @@ DEFAULT_INDEX_HTML = """\
 """
 
 
-def log_exception(request: Request, exc_info: t.Any) -> None:
+def log_exception(request: Request, exc_info: t.Any = True) -> None:
     """
     Logs an exception.  This is called by :meth:`handle_exception`
     if debugging is disabled and right before the handler is called.
@@ -105,11 +106,16 @@ class HTTPAppFactory(BaseAppFactory):
         enable_metrics: bool = Provide[
             BentoMLContainer.api_server_config.metrics.enabled
         ],
-    ) -> None:
+        max_concurrency: int | None = Provide[
+            BentoMLContainer.api_server_config.traffic.max_concurrency
+        ],
+    ):
         self.bento_service = bento_service
         self.enable_access_control = enable_access_control
         self.access_control_options = access_control_options
         self.enable_metrics = enable_metrics
+        timeout = BentoMLContainer.api_server_config.traffic.timeout.get()
+        super().__init__(timeout=timeout, max_concurrency=max_concurrency)
 
     @property
     def name(self) -> str:
@@ -203,9 +209,9 @@ class HTTPAppFactory(BaseAppFactory):
             middlewares.append(Middleware(middleware_cls, **options))
 
         if self.enable_access_control:
-            assert (
-                self.access_control_options.get("allow_origins") is not None
-            ), "To enable cors, access_control_allow_origin must be set"
+            assert self.access_control_options.get("allow_origins") is not None, (
+                "To enable cors, access_control_allow_origin must be set"
+            )
 
             from starlette.middleware.cors import CORSMiddleware
 
@@ -220,15 +226,15 @@ class HTTPAppFactory(BaseAppFactory):
             middlewares.append(Middleware(HTTPTrafficMetricsMiddleware))
 
         # otel middleware
-        import opentelemetry.instrumentation.asgi as otel_asgi
+        from opentelemetry.instrumentation.asgi import OpenTelemetryMiddleware
 
-        def client_request_hook(span: Span, _: dict[str, t.Any]) -> None:
+        def client_request_hook(span: Span, *_: t.Any) -> None:
             if span is not None:
                 trace_context.request_id = span.context.span_id
 
         middlewares.append(
             Middleware(
-                otel_asgi.OpenTelemetryMiddleware,
+                OpenTelemetryMiddleware,
                 excluded_urls=BentoMLContainer.tracing_excluded_urls.get(),
                 default_span_details=None,
                 server_request_hook=None,
@@ -256,14 +262,24 @@ class HTTPAppFactory(BaseAppFactory):
         return middlewares
 
     @property
-    def on_startup(self) -> list[t.Callable[[], None]]:
-        on_startup = [self.bento_service.on_asgi_app_startup]
+    def on_startup(self) -> list[LifecycleHook]:
+        on_startup = [
+            *map(with_app_arg, self.bento_service.startup_hooks),
+            with_app_arg(self.bento_service.on_asgi_app_startup),
+        ]
         if BentoMLContainer.development_mode.get():
             for runner in self.bento_service.runners:
-                on_startup.append(functools.partial(runner.init_local, quiet=True))
+                on_startup.append(
+                    with_app_arg(functools.partial(runner.init_local, quiet=True))
+                )
         else:
             for runner in self.bento_service.runners:
-                on_startup.append(runner.init_client)
+                if runner.embedded:
+                    on_startup.append(
+                        with_app_arg(functools.partial(runner.init_local, quiet=True))
+                    )
+                else:
+                    on_startup.append(with_app_arg(runner.init_client))
         on_startup.extend(super().on_startup)
         return on_startup
 
@@ -280,10 +296,13 @@ class HTTPAppFactory(BaseAppFactory):
         return PlainTextResponse("\n", status_code=200)
 
     @property
-    def on_shutdown(self) -> list[t.Callable[[], None]]:
-        on_shutdown = [self.bento_service.on_asgi_app_shutdown]
+    def on_shutdown(self) -> list[LifecycleHook]:
+        on_shutdown = [
+            *map(with_app_arg, self.bento_service.shutdown_hooks),
+            with_app_arg(self.bento_service.on_asgi_app_shutdown),
+        ]
         for runner in self.bento_service.runners:
-            on_shutdown.append(runner.destroy)
+            on_shutdown.append(with_app_arg(runner.destroy))
         on_shutdown.extend(super().on_shutdown)
         return on_shutdown
 
@@ -293,92 +312,93 @@ class HTTPAppFactory(BaseAppFactory):
             app.mount(app=mount_app, path=path, name=name)
         return app
 
-    @staticmethod
     def _create_api_endpoint(
-        api: InferenceAPI,
+        self,
+        api: InferenceAPI[t.Any],
     ) -> t.Callable[[Request], t.Coroutine[t.Any, t.Any, Response]]:
         """
-        Create api function for flask route, it wraps around user defined API
+        Create api function for starlette route, it wraps around user defined API
         callback and adapter class, and adds request logging and instrument metrics
         """
-        from starlette.responses import JSONResponse
         from starlette.concurrency import run_in_threadpool  # type: ignore
+        from starlette.responses import JSONResponse
+
+        from ..utils import is_async_callable
 
         async def api_func(request: Request) -> Response:
             # handle_request may raise 4xx or 5xx exception.
             output = None
-            try:
-                input_data = await api.input.from_http_request(request)
-                ctx = None
-                if asyncio.iscoroutinefunction(api.func):
+            with self.bento_service.context.in_request(request) as ctx:
+                try:
+                    input_data = await api.input.from_http_request(request)
                     if api.multi_input:
                         if api.needs_ctx:
-                            ctx = Context.from_http(request)
                             input_data[api.ctx_param] = ctx
-                        output = await api.func(**input_data)
-                    else:
-                        if api.needs_ctx:
-                            ctx = Context.from_http(request)
-                            output = await api.func(input_data, ctx)
+                        if is_async_callable(api.func):
+                            output = await api.func(**input_data)
                         else:
-                            output = await api.func(input_data)
-                else:
-                    if api.multi_input:
-                        if api.needs_ctx:
-                            ctx = Context.from_http(request)
-                            input_data[api.ctx_param] = ctx
-                        output: t.Any = await run_in_threadpool(api.func, **input_data)
+                            output = await run_in_threadpool(api.func, **input_data)
                     else:
+                        args = (input_data,)
                         if api.needs_ctx:
-                            ctx = Context.from_http(request)
-                            output = await run_in_threadpool(api.func, input_data, ctx)
+                            args = (input_data, ctx)
+                        if is_async_callable(api.func):
+                            output = await api.func(*args)
                         else:
-                            output = await run_in_threadpool(api.func, input_data)
+                            output = await run_in_threadpool(api.func, *args)
 
-                response = await api.output.to_http_response(output, ctx)
+                    response = await api.output.to_http_response(
+                        output, ctx if api.needs_ctx else None
+                    )
 
+                except BentoMLException as e:
+                    log_exception(request, sys.exc_info())
+                    if output is not None:
+                        import inspect
+
+                        signature = inspect.signature(api.output.to_proto)
+                        param = next(iter(signature.parameters.values()))
+                        ann = ""
+                        if param is not inspect.Parameter.empty:
+                            ann = param.annotation
+
+                        # more descriptive errors if output is available
+                        logger.error(
+                            "Function '%s' has 'input=%s,output=%s' as IO descriptor, and returns 'result=%s', while expected return type is '%s'",
+                            api.name,
+                            api.input,
+                            api.output,
+                            type(output),
+                            ann,
+                        )
+
+                    status = e.error_code.value
+                    if 400 <= status < 500 and status not in (401, 403):
+                        response = JSONResponse(
+                            content="BentoService error handling API request: %s"
+                            % str(e),
+                            status_code=status,
+                        )
+                    else:
+                        response = JSONResponse("", status_code=status)
+                except Exception:  # pylint: disable=broad-except
+                    # For all unexpected error, return 500 by default. For example,
+                    # if users' model raises an error of division by zero.
+                    log_exception(request, sys.exc_info())
+
+                    response = JSONResponse(
+                        "An error has occurred in BentoML user code when handling this request, find the error details in server logs",
+                        status_code=500,
+                    )
                 if trace_context.request_id is not None:
                     response.headers["X-BentoML-Request-ID"] = str(
                         trace_context.request_id
                     )
-            except BentoMLException as e:
-                log_exception(request, sys.exc_info())
-                if output is not None:
-                    import inspect
-
-                    signature = inspect.signature(api.output.to_proto)
-                    param = next(iter(signature.parameters.values()))
-                    ann = ""
-                    if param is not inspect.Parameter.empty:
-                        ann = param.annotation
-
-                    # more descriptive errors if output is available
-                    logger.error(
-                        "Function '%s' has 'input=%s,output=%s' as IO descriptor, and returns 'result=%s', while expected return type is '%s'",
-                        api.name,
-                        api.input,
-                        api.output,
-                        type(output),
-                        ann,
-                    )
-
-                status = e.error_code.value
-                if 400 <= status < 500 and status not in (401, 403):
-                    response = JSONResponse(
-                        content="BentoService error handling API request: %s" % str(e),
-                        status_code=status,
-                    )
-                else:
-                    response = JSONResponse("", status_code=status)
-            except Exception:  # pylint: disable=broad-except
-                # For all unexpected error, return 500 by default. For example,
-                # if users' model raises an error of division by zero.
-                log_exception(request, sys.exc_info())
-
-                response = JSONResponse(
-                    "An error has occurred in BentoML user code when handling this request, find the error details in server logs",
-                    status_code=500,
-                )
-            return response
+                if (
+                    BentoMLContainer.http.response.trace_id.get()
+                    and trace_context.trace_id is not None
+                ):
+                    response.headers["X-BentoML-Trace-ID"] = str(trace_context.trace_id)
+                return response
 
         return api_func
