@@ -49,6 +49,22 @@ def raise_deployment_config_error(err: BentoMLException, action: str) -> t.NoRet
     ) from None
 
 
+def convert_label_to_dict(label: tuple[str] | None) -> list[dict[str, str]] | None:
+    if label is None:
+        return None
+    collected_labels: list[dict[str, str]] = []
+    if label:
+        for item in label:
+            if "=" in item:
+                name, value = item.split("=", 1)
+            else:
+                raise CLIException(
+                    f"Invalid label format '{item}'. Expected format is key=value."
+                )
+            collected_labels.append({"key": name, "value": value})
+    return collected_labels
+
+
 def convert_env_to_dict(env: tuple[str] | None) -> list[dict[str, str]] | None:
     if env is None:
         return None
@@ -110,6 +126,12 @@ def convert_env_to_dict(env: tuple[str] | None) -> list[dict[str, str]] | None:
     ),
     help="Deployment strategy",
 )
+@click.option(
+    "--label",
+    type=click.STRING,
+    help="List of labels pass by --label key=value --label ...",
+    multiple=True,
+)
 @click.option(
     "--env",
     type=click.STRING,
@@ -158,6 +180,7 @@ def deploy_command(
     scaling_max: int | None,
     instance_type: str | None,
     strategy: str | None,
+    label: tuple[str] | None,
     env: tuple[str] | None,
     secret: tuple[str] | None,
     config_file: str | t.TextIO | None,
@@ -180,6 +203,7 @@ def deploy_command(
         instance_type=instance_type,
         strategy=strategy,
         env=env,
+        label=label,
         secret=secret,
         config_file=config_file,
         config_dict=config_dict,
@@ -261,7 +285,8 @@ def codespace(
 
         $ bentoml code --attach <codespace-name>
     """
-    import questionary
+    from rich.prompt import Prompt
+    from rich_toolkit.menu import Menu
 
     if attach and (env or secret):
         raise CLIException("Cannot specify both --attach and --env or --secret")
@@ -288,16 +313,16 @@ def codespace(
                 ]
             ]
 
-        chosen = questionary.select(
-            message="Select a codespace to attach to or create a new one",
-            choices=[{"name": d.name, "value": d} for d in deployments]
+        choice = Menu["t.Literal['new'] | Deployment"](
+            label="Select a codespace to attach to or create a new one",
+            options=[{"name": d.name, "value": d} for d in deployments]
             + [{"name": "Create a new codespace", "value": "new"}],
         ).ask()
 
-        if chosen == "new":
-            name = questionary.text(
+        if choice == "new":
+            name = Prompt.ask(
                 "Enter a name for the new codespace, or leave it blank to get a random name derived from the service"
-            ).ask()
+            )
             deployment = create_deployment(
                 bento=bento_dir,
                 name=name or None,
@@ -307,14 +332,12 @@ def codespace(
                 env=env,
                 secret=secret,
             )
-        elif chosen is None:
-            return
         else:
             if env or secret:
                 rich.print(
                     "[yellow]Warning:[/] --env and --secret are ignored when attaching to an existing codespace"
                 )
-            deployment = t.cast(Deployment, chosen)
+            deployment = choice
     deployment.watch(bento_dir)
 
 
@@ -362,6 +385,12 @@ def deployment_command():
     ),
     help="Deployment strategy",
 )
+@click.option(
+    "--label",
+    type=click.STRING,
+    help="List of labels pass by --label key=value --label ...",
+    multiple=True,
+)
 @click.option(
     "--env",
     type=click.STRING,
@@ -398,6 +427,7 @@ def update(  # type: ignore
     scaling_max: int | None,
     instance_type: str | None,
     strategy: str | None,
+    label: tuple[str] | None,
     env: tuple[str] | None,
     secret: tuple[str] | None,
     config_file: t.TextIO | None,
@@ -423,6 +453,7 @@ def update(  # type: ignore
         instance_type=instance_type,
         strategy=strategy,
         envs=convert_env_to_dict(env),
+        labels=convert_label_to_dict(label),
         secrets=list(secret) if secret else None,
         config_file=config_file,
         config_dict=cfg_dict,
@@ -483,6 +514,12 @@ def update(  # type: ignore
     ),
     help="Deployment strategy",
 )
+@click.option(
+    "--label",
+    type=click.STRING,
+    help="List of labels pass by --label key=value --label ...",
+    multiple=True,
+)
 @click.option(
     "--env",
     type=click.STRING,
@@ -508,6 +545,7 @@ def update(  # type: ignore
     help="Configuration json string",
     default=None,
 )
+@build_args_option
 @inject
 def apply(  # type: ignore
     bento: str | None,
@@ -518,6 +556,7 @@ def apply(  # type: ignore
     scaling_max: int | None,
     instance_type: str | None,
     strategy: str | None,
+    label: tuple[str] | None,
     env: tuple[str] | None,
     secret: tuple[str] | None,
     config_file: str | t.TextIO | None,
@@ -541,6 +580,7 @@ def apply(  # type: ignore
         scaling_min=scaling_min,
         instance_type=instance_type,
         strategy=strategy,
+        labels=convert_label_to_dict(label),
         envs=convert_env_to_dict(env),
         secrets=list(secret) if secret else None,
         config_file=config_file,
@@ -853,6 +893,7 @@ def create_deployment(
     scaling_max: int | None = None,
     instance_type: str | None = None,
     strategy: str | None = None,
+    label: tuple[str] | None = None,
     env: tuple[str] | None = None,
     secret: tuple[str] | None = None,
     config_file: str | t.TextIO | None = None,
@@ -875,6 +916,7 @@ def create_deployment(
         scaling_min=scaling_min,
         instance_type=instance_type,
         strategy=strategy,
+        labels=convert_label_to_dict(label),
         envs=convert_env_to_dict(env),
         secrets=list(secret) if secret else None,
         config_file=config_file,
